<?php

App::uses('EvCheckoutAppController', 'EvCheckout.Controller');
App::uses('OrderStatus', 'EvCheckout.Model');
/**
 * Checkout Orders Controller
 *
 * @package EvCheckout/Orders
 */
class OrdersController extends EvCheckoutAppController {

	public $components = array(
		'Transactions.Transactions', 
		'Transactions.SagepayDirect'
	);

	public function beforeFilter() {

		if(Configure::read('enableSSL')) {

			$this->Security->blackHoleCallback = 'forceSSL';
			$this->Security->requireSecure(
				'invoice'
			);
			
		}

		$this->adminActions[] = 'admin_print';

		$this->Auth->allow(array(
			'basket',
			'register', 
			'remove_item'
		));

		parent::beforeFilter();

	}

	function forceSSL() { 
		$this->redirect('https://' . $_SERVER['SERVER_NAME'] . $this->here); 
	} 

	/**
	 * Defines the fields displayed in a filter form.
	 *
	 * Defaults to id, display name, is_active, created and modified fields.
	 *
	 * Override in your controller to customise.
	 *
	 * Format:
	 *
	 * 'id' => array(
	 *         'label' => label text,
	 *         'type' => data type of control
	 *         'compare' => SQL query. e.g. Model.field='%s' (where %s is replace by the value of the field)
	 *         'default' => value you want the data filtering by by default
	 * )
	 */
	protected function _adminFilterFields() {

		$Model = $this->{$this->modelClass};
		$modelAlias = $Model->alias;

		$fields = parent::_adminFilterFields(); 

		unset($fields[$Model->alias.'.name']);
		unset($fields[$Model->alias.'.created']);
		unset($fields[$Model->alias.'.modified']);

		$fields[$Model->alias.'.id']['label'] = 'Order Ref';

		$fields[$Model->alias.'.order_status_id'] = array(
			'label' => 'Order Status', 
			'type' => 'select', 
			'compare' => array(
				$Model->alias.'.order_status_id' => '%s'
			)
		);

		$fields['Customer.email'] = array(
			'label' => 'Customer Email', 
			'type' => 'string', 
			'compare' => array(
				'Customer.email LIKE' => '%%%s%%'
			)
		);

		$fields['Customer.last_name'] = array(
			'label' => 'Customer Surname', 
			'type' => 'string', 
			'compare' => array(
				'Customer.last_name LIKE' => '%%%s%%'
			)
		);

		$fields[$Model->alias.'.order_time_from'] = array(
			'label' => 'Placed After', 
			'type' => 'daterange_from', 
			'compare' => array(
				$Model->alias.'.order_time >' => '%s'
			)
		);

		$fields[$Model->alias.'.order_time_to'] = array(
			'label' => 'Placed Before', 
			'type' => 'daterange_to', 
			'compare' => array(
				$Model->alias.'.order_time <' => '%s'
			)
		);

		return $fields;

	}

	protected function _adminIndexColumns() {

		$Model = $this->{$this->modelClass};

		return array(
			$Model->alias.'.id' => array(
				'type' => 'integer',
				'null' => false,
				'default' => null,
				'length' => (int) 11,
				'key' => 'primary',
				'label' => 'Order Ref'
			),
			$Model->alias.'.order_time' => array(
				'type' => 'datetime',
				'null' => true,
				'default' => null,
				'length' => null,
				'label' => 'Order Placed'
			),
			'Customer.first_name' => array(
				'type' => 'string',
				'label' => 'First Name'
			),
			'Customer.last_name' => array(
				'type' => 'string',
				'label' => 'Last Name'
			),
			'Customer.email' => array(
				'type' => 'string',
				'label' => 'Email'
			),
			'OrderStatus.name' => array(
				'type' => 'string',
				'label' => 'Status'
			),
			$Model->alias.'.total' => array(
				'type' => 'currency',
				'label' => 'Total'
			),
			$Model->alias.'.modified' => array(
				'type' => 'datetime',
				'null' => true,
				'default' => null,
				'length' => null,
				'label' => 'Last Modified'
			),
		);

	}

	protected function _adminPopulateLookups() {

		$Model = $this->{$this->modelClass}; 

		parent::_adminPopulateLookups();

		$this->set('orderStatuses', $Model->OrderStatus->find('list'));

	}

	/**
	 * Defines the buttons in the toolbar displayed on an admin_form
	 *
	 * Override in your controller to customise
	 */
	protected function _adminFormToolbar($id = null) {

		$toolbar = array();

		if($id !== null) {

			$toolbar['Print'] = array(
				'url' => array(
					'admin' => false,
					'plugin' => 'ev_checkout', 
					'controller' => 'orders',
					'action' => 'receipt', 
					$id
				)
			);

		}

		return $toolbar;

	}


	protected function _adminIndexActions() {

		$actions = array(
			'Edit'
		);

		return $actions;

	}

	protected function _adminIndexToolbar() {

		return array();

	}

	/**
	 * Pagination settings for admin_index
	 *
	 * Override in your own controller to customise
	 *
	 * @return array
	 */
	protected function _adminIndexPaginate() {

		$Model = $this->{$this->modelClass};

		$query = parent::_adminIndexPaginate(); 

		$query['conditions'][$Model->alias.'.order_status_id <>'] = OrderStatus::IN_BASKET;

		$query['contain'][] = 'Customer';
		$query['contain'][] = 'OrderStatus';

		return $query; 

	}


	public function admin_edit($id = null) {

		//No add so redirect if $id is null 
		if($id == null) {
			
			$this->Session->setFlash(array(
				'title' => 'Cannot find order',
				'description' => 'Could not find order, please search for the order in the listing below.'
			), 'flash_fail');

			$this->redirect(array('action' => 'index')); 

		}

		parent::admin_edit($id);

		// get and assign the previous status, this is used afterSave to check
		// whether a unit sold increment is required on a product
		if (!empty($this->request->data['Order']['previousStatus'])) {

			$previousStatus = $this->request->data['Order']['previousStatus'];

		} else {

			$previousStatus = $this->request->data['Order']['order_status_id'];

		}
		$this->set('previousStatus', $previousStatus);
		$this->view = 'admin_form';

	}

	public function admin_print($id = null) {

		$this->view($id); 

		$this->view = 'view';
		$this->layout = 'print';

		$this->set('title_for_layout', 'Print Order | '.Configure::read('SiteSetting.site_title'));

	}


	public function admin_resendConfirmationEmail($id) {
		$this->Order->emailConfirmation($id);
		$this->Session->setFlash('Email Sent', 'flash_success');
		return $this->redirect(array('action'=>'edit', $id));
	}


	public function admin_confirmationemail($id) {
		$order = $this->Order->readForEdit($id);
		$this->view = 'confirmation';
		$this->set(array(
			'order' => $order,
			'delayed' => false,
			'telephone' => Configure::read('SiteSetting.telephone_number'),
			'sortCode' => Configure::read('SiteSetting.bank_sortcode'),
			'accountNumber' => Configure::read('SiteSetting.bank_account')
		));
		
	}


/*
 * BASKET AND CHECKOUT METHODS  
 */

/**
 * Displays the cart 
 */
	public function basket() {

		$Model = $this->{$this->modelClass};
		$this->set('cart', $Model->getCart());
		$this->set('continueShoppingUrl', $this->_getContinueShoppingUrl());
		$this->set('title_for_layout', 'Basket | ' . Configure::read('SiteSetting.site_title'));

	}


/** 
 * Gets the continue shopping url, this will either be defined by the Session EvCheckout.lastListingPage or the default listing url ("/")
 */
	protected function _getContinueShoppingUrl() {

		if ( $this->Session->read('EvCheckout.lastListingPage') ) {

			return $this->Session->read('EvCheckout.lastListingPage'); 

		} else {

			return "/";

		}

	}


/**
 * Removes an item from the cart which has the given item ID
 * @param  int $itemId OrderItem::id
 */
	public function remove_item($itemId) {
		$Model = $this->{$this->modelClass};
		if ($Model->removeItem($itemId)) {
			$Model->updateOrderTotal();
			$this->Session->setFlash('Item removed', 'flash_success');
		}
		else {
			$this->Session->setFlash('Item cannot be removed', 'flash_fail');
		}
		$this->redirect(array('action' => 'basket')); 
	}


/**
 * Handles user login and registration
 */
	public function register() {

		$Model = $this->{$this->modelClass};

		//If user is logged in skip this step and move onto the next step
		$user = $this->Auth->user();
		
		if($user) {

			if(Configure::read('EvCheckout.onepage') == true) {

				return $this->redirect(array(
					'action' => 'onepage'
				));

			} else {

				return $this->redirect(array(
					'action' => 'delivery'
				));

			}

		}

		//Else show the login and registration page. 
		//We setup the login redirect here because if the customer chooses to login that 
		// is handled by the UsersController
		$this->Session->write('Auth.redirect', 
			Router::url(array(
				'action' => 'delivery'
			))
		);

		$Model = $this->{$this->modelClass};

		//User has submitted registration or login form
		if(! empty($this->request->data['Customer'])) {

			//Process user registration 
			$userRegistrationData = $this->request->data; 

			//Check the data validates 
			if($Model->Customer->register($userRegistrationData)) {

				$userId = $Model->Customer->id; 

				//Sign in user and send to delivery page
				$fullUser = $Model->Customer->getUserForLogin($userId);

				//Log user in and link order
				$this->Auth->login($fullUser);
				$Model->setUser($fullUser['User']['id']);

				return $this->redirect($this->Auth->redirect());

			} else {

				$this->set('errors', $Model->Customer->validationErrors);

			}

		}

		$this->set('title_for_layout', 'Register'. ' | ' . Configure::read('SiteSetting.site_title'));
		
	}


/**
 * Processes a delivery address passed in from request->data['DeliveryAddress']
 * Handles either setting order address to previous address or adding a new one and attaching to order
 * @return boolean True on success
 */
	protected function _processDeliveryAddress() {

		$Model = $this->{$this->modelClass};

		if(! empty($this->request->data)) {

			$userId = $this->Auth->User('User.id');
			$orderId = $this->Order->getCartId();

			if(! empty($this->request->data['DeliveryAddress']['id'])) {

				return $Model->setDeliveryAddress($this->request->data['DeliveryAddress']['id']);

			} else {

				return $Model->DeliveryAddress->createAddress($userId, $this->request->data, $orderId);

			}

		} 

		return false; 

	}



	public function paymentCallback($orderId) {

		$Model = $this->{$this->modelClass};

		$this->SagepayDirect->setup();
		$transaction = $this->SagepayDirect->process3dSecure();

		debug($transaction);
	
		if (isset($transaction['result']) && $transaction['result']) {

			$Model->markAsPaid($orderId);
			
			//Payment taken 
			$this->redirect(
				array(
					'action' => 'success', 
					$orderId
				)
			);

		} else {

			$this->Session->setFlash('Processing your payment failed, please check your details and try again.', 'flash_fail');

			//Payment taken 
			$this->redirect(
				array(
					'action' => 'invoice'
				)
			);

		}

	}

/**
 * Proceses OnSite credit card payments
 * @return [type] [description]
 */
	protected function _processCreditCard() {
		
		$Model = $this->{$this->modelClass};

		//If credit card information is not set we cannot process
		if(empty($this->request->data['CreditCard'])) {
			return false; 
		}

		//Split name into first and second 
		$fullName = $this->request->data['CreditCard']['name'];
		$splitName = explode(" ", $fullName);
		$this->request->data['CreditCard']['last_name'] = array_pop($splitName);

		if(count($splitName)) {

			$this->request->data['CreditCard']['first_name'] = implode(" ", $splitName);
		
		} else {
		
			$this->request->data['CreditCard']['first_name'] = '';
		
		}


		//Check that card validates 
		$this->loadModel('EvCheckout.CreditCard');
		$this->CreditCard->set($this->request->data);
		if($this->CreditCard->validates()) {
		
			$cart = $Model->getCart();
			$transaction = $this->Transactions->takePayment(
				array(
					'return' => Router::url(array(
						'action' => 'paymentCallback', 
						$cart['Order']['id']
					), true),
					'cancel' => null
				), 
				array(
					'model' => 'EvCheckout.Order', 
					'model_id' => $cart['Order']['id']
				), 
				$cart['Order']['total'],
				array(

				), 
				array(
					'card' => $this->request->data['CreditCard'],
					'address' => $cart['InvoiceAddress'], 
					'user' => $this->Auth->user('User')
				), 
				$cart['Order']['user_id']
				// 'sagepay_direct'
			);

			if (isset($transaction['result']) && $transaction['result']) {

				$Model->markAsPaid($cart['Order']['id']);
				return true;

			} else {

				$this->Session->setFlash('Processing your payment failed, please check your details and try again.', 'flash_fail');

			}

		}

		return false;
	}

	protected function _processOffsitePayment() {

	}

	protected function _processOnsitePayment() {

		return $this->_processCreditCard(); 

	}

	protected function _processInvoiceAddress() {

		$invoiceAddressSaved = false;

		if(! empty($this->request->data)) {

			$Model = $this->{$this->modelClass}; 

			$orderId = $Model->getCartId();
			$userId = $this->Auth->user('User.id');

			$cart = $Model->getCart();

			//Either mark delivery address as invoice as well or create new address
			if(
				isset($this->request->data['InvoiceAddress']['match_delivery_address']) 
				&& $this->request->data['InvoiceAddress']['match_delivery_address'] == 1
			) {

				$Model->setInvoiceAddress($cart['DeliveryAddress']['id']);
				$invoiceAddressSaved = true;

			} else if (! empty($this->request->data['InvoiceAddress']['invoice_address_id'])) {

				$Model->setInvoiceAddress($this->request->data['InvoiceAddress']['invoice_address_id']);
				$invoiceAddressSaved = true;
		
			} else {

				if($Model->InvoiceAddress->createAddress($userId, $this->request->data, $orderId)) {

					$invoiceAddressSaved = true;

				}

			}

		}

		return $invoiceAddressSaved;

	}


	public function delivery() {

		if($this->_processDeliveryAddress()) {

			$this->redirect(array(
				'action' => 'invoice'
			));

		}

		$this->_populatePreviousDeliveryAddresses();
		$this->set('title_for_layout', 'Delivery'. ' | ' . Configure::read('SiteSetting.site_title'));


	}

	public function invoice() {

		$Model = $this->{$this->modelClass};
		$cart = $this->_populateCart();

		$this->_populateLookupsInvoice();

		if($this->_processInvoiceAddress() & $this->_processCreditCard()) {

			//Payment taken 
			$this->redirect(
				array(
					'action' => 'success', 
					$cart['Order']['id']
				)
			);

		}

		$this->_populatePreviousInvoiceAddresses();
		$this->set('title_for_layout', 'Card Details | ' . Configure::read('SiteSetting.site_title'));
		
	}


	public function payment() {
	
		$data = $this->Order->getCart();

		if (empty($data['Order']['id'])) {

			return $this->redirect(array(
				'action' => 'basket'
			));

		}

	}

	public function success($id) {

		$Model = $this->{$this->modelClass};
		$this->set('cart', $Model->readForView($id));

		//Get the content for the page 
		if(Configure::read('EvCheckout.confirmation_page_id')) {

			$Page = ClassRegistry::init('Page');
			$data = $Page->readForView(Configure::read('EvCheckout.confirmation_page_id'));

			$this->set('data', $data);

		}

		$this->set('title_for_layout', 'Order Confirmation | ' . Configure::read('SiteSetting.site_title'));

	}

	public function onepage() {

		$Model = $this->{$this->modelClass};
		$cart = $Model->getCart();

		$this->_populateCart();
		$this->_populateLookupsInvoice();

		$this->_processDeliveryAddress();

		$this->_processInvoiceAddress(); 

		if($this->_processCreditCard()) {

			//Payment taken 
			$this->redirect(
				array(
					'action' => 'success', 
					$cart['Order']['id']
				)
			);

		}

		$this->set('title_for_layout', 'Checkout | ' . Configure::read('SiteSetting.site_title'));

	}

	public function view($id) {

		$Model = $this->{$this->modelClass};

		$cart = $Model->readForView($id);

		$this->set('data', $cart);
		$this->set('title_for_layout', 'Order Summary | '.Configure::read('SiteSetting.site_title'));

	}

	public function receipt($id) {

		$this->view($id); 

		$this->view = 'view';
		$this->layout = 'print';

		$this->set('title_for_layout', 'Print Order | '.Configure::read('SiteSetting.site_title'));

	}


	protected function _populateCart() {

		$Model = $this->{$this->modelClass};
		$cart = $Model->getCart();
		$this->set('cart', $cart);

		return $cart;

	}

	protected function _populatePreviousInvoiceAddresses() {

		$Model = $this->{$this->modelClass};
		$userId = $this->Auth->User('User.id');
		$invoice_addresses = $Model->InvoiceAddress->recentInvoiceAddresses($userId, 5);
		$this->set(compact('invoice_addresses'));

	}

	protected function _populatePreviousDeliveryAddresses() {

		$Model = $this->{$this->modelClass};
		$userId = $this->Auth->User('User.id');
		$delivery_addresses = $Model->DeliveryAddress->recentDeliveryAddresses($userId, 5);
		$this->set(compact('delivery_addresses'));

	}

	protected function _populateLookupsInvoice() {

		//Available months 
		$months = $this->_getMonthsList();
		$this->set('expiryMonths', $months);
		$this->set('startMonths', $months);
		//Expiry months
		$this->set('expiryYears', $this->_getExpiryYearsList()); 
		//Start months
		$this->set('startYears', $this->_getStartYearsList());
	
	}

	protected function _getMonthsList() {

		$months = array_map(array($this, '_padArray'), range(1, 12));
		return array_combine($months, $months);
	
	}

	protected function _getExpiryYearsList() {

		$expiryYears = range(date("y"), date("y")+10);
		$expiryYearsLong = range(date("Y"), date("Y")+10);
		return array_combine($expiryYears, $expiryYearsLong);

	}

	protected function _getStartYearsList() {

		$startYears = range(date("y"), date("y")-10);
		$startYearsLong = range(date("Y"), date("Y")-10);
		return array_combine($startYears, $startYearsLong);

	}

	/**
	 * Used to pad out month and year arrays from 1 to 01
	 * @param  mixed 	$number [description]
	 * @return string 	Padded array
	 */
	protected function _padArray($number) {
		return str_pad( $number, 2, "0", STR_PAD_LEFT);
	}


}